import { Collection, Message } from "discord.js";
import { openai } from "./index";
import { formatMessage } from "./toOpenAIMessages";

export default class Moderation {
  /** Represents cache of messages that have been checked aganist OpenAI moderation API. */
  private static cache = new Collection<string, boolean>();

  public static async checkMessage(message: Message): Promise<boolean> {
    if (this.cache.has(message.id)) {
      return this.cache.get(message.id) as boolean;
    }

    const warningReaction = message.reactions.resolve("⚠");
    // if bot reacted to that message, we already know that it returned true for moderation API
    if (warningReaction && warningReaction.me) {
      this.cache.set(message.id, true);
      return true;
    }
    
    try{
      const answer = await openai.createModeration({
        input: formatMessage(message),
      });

      const flagged = answer.data.results[0].flagged;
      this.cache.set(message.id, flagged);
      if (flagged) message.react("⚠");

      return flagged;
    }
    catch (e) {
      console.error(`Error occured while handling moderation request (${(e as object).constructor.name}):`);
      console.error(e);
      return false;
    }
  }
}
